//
//  SWPrimitives.h
//  SWGameLib
//
//
//  Copyright (c) 2010 Sangwoo Im
//
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
//
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
//
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  THE SOFTWARE.
//  
//  Created by Sangwoo Im on 9/15/10.
//  Copyright 2010 Sangwoo Im. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "ccTypes.h"

/**
 * Draws an anti-aliased line with two different colors assigned to head and tail respectively.
 *
 * Warning: this function does not care about the current opengl context. configure and reset
 * the context manually.
 *
 * @param head   the position where the line drawing begins
 * @param tail   the position where the line drawing ends
 * @param width  the line width
 * @param colorH the color for head 
 * @param colorT the color for tail
 */
void swDrawLineCC(CGPoint head, CGPoint tail, CGFloat width, ccColor4B colorH, ccColor4B colorT);
/**
 * Draws an anti-aliased line with one color.
 *
 * @param head  the position where the line drawing begins.
 * @param tail  the position where the line drawing ends.
 * @param width the line width
 * @param color the color of the line to be drawn.
 */
void swDrawLineC(CGPoint head, CGPoint tail, CGFloat width, ccColor4B color);
/**
 * Draws a polygon with anti-aliased lines.
 *
 * @param poli           an array of the points to represent a polygon.
 * @param width          the line width
 * @param numberOfPoints the number of points given by "poli"
 * @param color          the color of the polygon.
 */
void swDrawPoly(CGPoint *poli, CGFloat width, NSUInteger numberOfPoints, ccColor4B color);
/**
 * Draws a circle with anti-aliased edges.
 *
 * @param pos            the center of the circle.
 * @param radius         radius
 * @param width          width
 * @param numberOfPoints the number of points to be used for circle drawing.
 * @param color          the color of the circle.
 */
void swDrawCircle(CGPoint pos, CGFloat radius, CGFloat width, NSUInteger numberOfPoints, ccColor4B color);